import 'package:collection/collection.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:panda/controllers/panda.controller.dart';
import 'package:panda/cubit/panda.cubit.dart';
import 'package:panda/cubit/panda.state.dart';
import 'package:panda/models/view-config.model.dart';
import 'package:panda/utils/transition.utils.dart';
import 'package:panda/widgets/section-navigator.dart';

// Multi Section Layout expects all top level view configs to be of type group.
class PandaMultiSectionLayout extends StatefulWidget {
  final PandaController controller;

  const PandaMultiSectionLayout({Key? key, required this.controller}) : super(key: key);

  @override
  _PandaMultiSectionLayoutState createState() => _PandaMultiSectionLayoutState();
}

class _PandaMultiSectionLayoutState extends State<PandaMultiSectionLayout> {
  @override
  void initState() {
    super.initState();
    widget.controller.onNavigateToSection.listen((int sectionIndex) async {
      _openSectionNavigator(sectionIndex);
    });
  }

  @override
  Widget build(BuildContext context) {
    return BlocBuilder<PandaCubit, PandaState>(
      builder: (_, state) {
        return Column(
          children: (state.formDescriptor?.viewConfigs ?? [])
              .mapIndexed((index, viewConfig) => _buildTopLevelGroupRow(state, index, viewConfig))
              .toList()
              .expand((child) sync* {
                yield const Divider(color: Color(0xFFD6D6D6));
                yield child;
              })
              .skip(1)
              .toList(),
        );
      },
    );
  }

  Widget _buildTopLevelGroupRow(PandaState state, int index, PandaViewConfigModel<dynamic> viewConfig) {
    return ListTile(
      contentPadding: EdgeInsets.zero,
      leading: CircleAvatar(
        backgroundColor: Colors.lightBlueAccent.withOpacity(0.2),
        child: Text(
          "${index + 1}",
          style: const TextStyle(
            fontSize: 18.0,
            fontWeight: FontWeight.w800,
            color: Colors.blueAccent,
          ),
        ),
      ),
      title: Text(
        viewConfig.title ?? "",
        style: const TextStyle(
          fontWeight: FontWeight.bold,
        ),
      ),
      subtitle: (viewConfig.description != null) ? Text(viewConfig.description!) : null,
      trailing: _buildCompletedIcon(state, viewConfig),
      onTap: () => _openSectionNavigator(index),
    );
  }

  Widget _buildCompletedIcon(PandaState state, PandaViewConfigModel<dynamic> viewConfig) {
    Color? color;
    IconData icon;
    bool showBackground;
    if (state.formResult?.isSectionFilled(viewConfig.keyName) ?? false) {
      color = Theme.of(context).primaryColor;
      icon = Icons.check;
      showBackground = true;
    } else {
      color = Colors.grey;
      icon = Icons.edit;
      showBackground = false;
    }

    return SizedBox(
      width: 40.0,
      height: 40.0,
      child: Stack(
        children: <Widget>[
          if (showBackground)
            Positioned.fill(
              child: Container(
                decoration: BoxDecoration(
                  color: color.withOpacity(0.2),
                  borderRadius: BorderRadius.circular(32.0),
                ),
              ),
            ),
          Center(
            child: Padding(
              padding: const EdgeInsets.all(10.0),
              child: Text(
                String.fromCharCode(icon.codePoint),
                style: TextStyle(
                  fontFamily: icon.fontFamily,
                  package: icon.fontPackage,
                  fontSize: 18.0,
                  color: color.withOpacity(0.8),
                ),
              ),
            ),
          ),
        ],
      ),
    );
  }

  void _openSectionNavigator(int index) {
    Navigator.of(context).push(
      PageRouteBuilder<PandaSectionNavigator>(
        opaque: false,
        transitionDuration: const Duration(milliseconds: 500),
        transitionsBuilder: PandaTransitionUtils.slideUpTransition,
        pageBuilder: (_, __, ___) => BlocProvider.value(
          value: PandaCubit.of(context),
          child: PandaSectionNavigator(index),
        ),
      ),
    );
  }
}
