import 'dart:async';
import 'dart:io';

import 'package:path_provider/path_provider.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:uuid/uuid.dart';

class PandaFileStorage {
  static final PandaFileStorage _singleton = PandaFileStorage._internal();

  static PandaFileStorage getInstance() => _singleton;

  PandaFileStorage._internal();

  static String _getKey(String fileId) {
    return "file-storage/$fileId";
  }

  Future<String?> _getFilePathFromKey(String key) async {
    final prefs = await SharedPreferences.getInstance();
    if (!prefs.containsKey(key)) {
      return null;
    }
    return prefs.getString(key);
  }

  Future<String?> getFilePath(String fileId) async {
    final key = _getKey(fileId);
    return _getFilePathFromKey(key);
  }

  Future<File?> _getFileFromKey(String key) async {
    final filePath = await _getFilePathFromKey(key);

    if (filePath == null) {
      return null;
    }

    final file = File(filePath);

    if (!(await file.exists())) {
      return null;
    }

    return file;
  }

  Future<File?> getFile(String fileId) async {
    final filePath = await getFilePath(fileId);

    if (filePath == null) {
      return null;
    }

    final file = File(filePath);

    if (!(await file.exists())) {
      return null;
    }

    return file;
  }

  Future<String> generateFilePath(String fileId) async {
    final key = _getKey(fileId);

    final prefs = await SharedPreferences.getInstance();
    String? filePath = prefs.getString(key);
    if (filePath != null) {
      return filePath;
    }

    final uuid = (const Uuid()).v4();
    Directory directory = await getApplicationDocumentsDirectory();
    filePath = "${directory.path}/$uuid";
    prefs.setString(key, filePath);

    return filePath;
  }

  Future<void> storeFile(String fileId, File file) async {
    final filePath = await (generateFilePath(fileId));
    await file.copy(filePath);
  }

  Future<void> _deleteFileFromKey(String key) async {
    final file = await _getFileFromKey(key);
    if (file != null) {
      await file.delete();
    }
    final prefs = await SharedPreferences.getInstance();
    if (prefs.containsKey(key)) {
      prefs.remove(key);
    }
  }

  Future<void> deleteFile(String fileId) async {
    final file = await getFile(fileId);
    if (file != null) {
      await file.delete();
    }
    final prefs = await SharedPreferences.getInstance();
    if (prefs.containsKey(_getKey(fileId))) {
      prefs.remove(_getKey(fileId));
    }
  }

  Future<void> clear() async {
    final prefs = await SharedPreferences.getInstance();
    for (String key in prefs.getKeys()) {
      if (key.startsWith("file-storage/")) {
        await _deleteFileFromKey(key);
      }
    }
  }

  Future<void> clearPictures(List<String> picturesToRemove) async {
    final prefs = await SharedPreferences.getInstance();
    for (String key in prefs.getKeys()) {
      if (key.startsWith("file-storage/")) {
        for (int i = 0; i < picturesToRemove.length; i++) {
          if (key == "file-storage/" + picturesToRemove[i]) {
            await _deleteFileFromKey(key);
          }
        }
      }
    }
  }
}
