import 'dart:collection';

import 'package:panda/models/form-validation.model.dart';
import 'package:panda/models/validation-config/type-config/max-length-validator.type-config.dart';
import 'package:panda/models/validation-config/type-config/min-length-validator.type-config.dart';
import 'package:panda/models/validation-config/type-config/range-validator.type-config.dart';
import 'package:panda/models/validation-config/type-config/required-validator.type-config.dart';
import 'package:panda/models/validation-config/validation-error.model.dart';
import 'package:panda/models/validation-config/validator-type.enum.dart';
import 'package:panda/models/validation-config/validator.model.dart';
import 'package:panda/utils/string.utils.dart';

class PandaValidatorConfigUtils {
  static List<PandaValidationError> validate(PandaValidatorConfig validator, LinkedHashMap<String, dynamic>? model) {
    switch (validator.type) {
      case PandaValidatorTypeEnum.MaxLength:
        return _validateMaxLength(validator, model);
      case PandaValidatorTypeEnum.MinLength:
        return _validateMinLength(validator, model);
      case PandaValidatorTypeEnum.Required:
        return _validateRequired(validator, model);
      case PandaValidatorTypeEnum.Range:
        return _validateRange(validator, model);
      default:
        return [];
    }
  }

  static bool hasError(PandaFormValidation? validation) {
    if ((validation?.errors ?? []).isNotEmpty) {
      return true;
    }

    for (String key in validation!.values!.keys) {
      if (PandaValidatorConfigUtils.hasError(validation.values![key])) {
        return true;
      }
    }

    return false;
  }

  // Looks through the form validation to find the validation errors matching the key. The key should be a series
  // of names separated by a '.', e.g. a.b.c. If nothing matches the key, returns [].
  static PandaFormValidation? findFormValidation(List<String> keyTree, PandaFormValidation? validation) {
    PandaFormValidation? deepValidation = validation;
    for (String segment in (keyTree)) {
      // Values should never be null here and we always make sure to have inserted the validation before moving deeper.
      deepValidation!.values!.putIfAbsent(
        segment,
        () => PandaFormValidation(
          errors: [],
          values: LinkedHashMap<String, PandaFormValidation>(),
        ),
      );

      deepValidation = deepValidation.values![segment];
    }

    return deepValidation;
  }

  static List<PandaValidationError> _validateMaxLength(
    PandaValidatorConfig validator,
    LinkedHashMap<String, dynamic>? model,
  ) {
    final config = validator.typeConfig as PandaMaxLengthValidatorTypeConfig;
    if (config.maxLength == null || config.maxLength is! int) {
      return [];
    }

    if ((model?.containsKey("value") ?? false) &&
        model!["value"] is String &&
        (model["value"] as String).length > config.maxLength!) {
      return [
        PandaValidationError(
          error: validator.error,
        ),
      ];
    }

    return [];
  }

  static List<PandaValidationError> _validateMinLength(
    PandaValidatorConfig validator,
    LinkedHashMap<String, dynamic>? model,
  ) {
    final config = validator.typeConfig as PandaMinLengthValidatorTypeConfig;
    if (config.minLength == null || config.minLength is! int || config.minLength! <= 0) {
      return [];
    }

    if ((model?.containsKey("value") ?? false) &&
        model!["value"] is String &&
        (model["value"] as String).length < config.minLength!) {
      return [
        PandaValidationError(
          error: validator.error,
        ),
      ];
    }

    return [];
  }

  static List<PandaValidationError> _validateRequired(
    PandaValidatorConfig validator,
    LinkedHashMap<String, dynamic>? model,
  ) {
    final config = validator.typeConfig as PandaRequiredValidatorTypeConfig;
    if (config.required &&
        (!(model?.containsKey("value") ?? false) ||
            model!["value"] == null ||
            (model["value"] is String && (model["value"] as String).isEmpty))) {
      return [
        PandaValidationError(
          error: validator.error,
        ),
      ];
    }

    return [];
  }

  static List<PandaValidationError> _validateRange(
    PandaValidatorConfig validator,
    LinkedHashMap<String, dynamic>? model,
  ) {
    final config = validator.typeConfig as PandaRangeValidatorTypeConfig;
    if (config.min == null || config.min is! int || config.max == null || config.max is! int) {
      return [];
    }

    if ((model?.containsKey("value") ?? false) &&
        model!["value"] is String &&
        (!PandaStringUtils.isInt(model["value"]) ||
            (int.tryParse(model["value"] as String)! < config.min! ||
                int.tryParse(model["value"] as String)! > config.max!))) {
      return [
        PandaValidationError(
          error: validator.error,
        ),
      ];
    }

    return [];
  }
}
