import 'dart:collection';
import 'dart:io';

import 'package:carousel_slider/carousel_slider.dart';
import 'package:collection/collection.dart';
import 'package:flutter/material.dart';
import 'package:nice_flutter_kit/nice_flutter_kit.dart';
import 'package:panda/utils/file.storage.dart';
import 'package:panda/utils/picture.utils.dart';
import 'package:panda/widgets/base-widget-state.dart';
import 'package:panda/widgets/base-widget.dart';
import 'package:rxdart/rxdart.dart';

class PandaPictureWidget extends PandaBaseWidget {
  const PandaPictureWidget(List<String> keyTree, {Key? key}) : super(keyTree, key: key);

  @override
  State<StatefulWidget> createState() => _PandaPictureWidgetState();
}

class _PandaPictureWidgetState extends PandaBaseWidgetState<PandaPictureWidget, void> {
  late BehaviorSubject<List<String?>?> _imageLinks;
  late BehaviorSubject<bool> _isLoading;

  @override
  void dispose() {
    _imageLinks.close();
    _isLoading.close();
    super.dispose();
  }

  @override
  void didChangeDependencies() async {
    super.didChangeDependencies();

    _imageLinks = BehaviorSubject<List<String>>();
    _isLoading = BehaviorSubject<bool>.seeded(false);

    // Get download urls for images.
    final List<dynamic> fileIds = ((node?.value ?? const {})["value"] as List).where((id) => id != null).toList();
    List<String?> imageLinks;

    if (fileIds.isNotEmpty) {
      imageLinks = List.filled(fileIds.length, null);
      await Future.wait(
        fileIds.mapIndexed((int index, dynamic fileId) async {
          if (fileId is! String) {
            return;
          }
          if (!fileId.startsWith("offline")) {
            imageLinks[index] = await getDownloadUrl(fileId);
          } else {
            imageLinks[index] = await PandaFileStorage.getInstance().getFilePath(fileId);
          }
        }),
      );
    } else {
      imageLinks = [];
    }

    if (!_imageLinks.isClosed) {
      _imageLinks.sink.add(imageLinks);
    }
  }

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.symmetric(vertical: 8.0),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.stretch,
        children: <Widget>[
          if (viewConfig?.title != null)
            Padding(
              padding: const EdgeInsets.symmetric(vertical: 10.0),
              child: Text(
                viewConfig!.title!,
                style: const TextStyle(
                  fontWeight: FontWeight.bold,
                ),
              ),
            ),
          StreamBuilder<bool>(
            stream: _isLoading.stream,
            initialData: false,
            builder: (context, AsyncSnapshot<bool> snapshot) {
              if (snapshot.data!) {
                return _buildLoading();
              }

              return _buildImages();
            },
          ),
        ],
      ),
    );
  }

  Widget _buildLoading() {
    return const NiceLoadingOverlay(
      loading: true,
      child: SizedBox(
        height: 200.0,
        width: 200.0,
      ),
    );
  }

  Widget _buildImages() {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.stretch,
      children: <Widget>[
        StreamBuilder<List<String?>?>(
          stream: _imageLinks.stream,
          initialData: const [],
          builder: (context, AsyncSnapshot<List<String?>?> snapshot) {
            return Column(
              children: <Widget>[
                if ((snapshot.data ?? []).isNotEmpty)
                  Padding(
                    padding: const EdgeInsets.symmetric(vertical: 8.0),
                    child: CarouselSlider(
                      options: CarouselOptions(
                        height: 200,
                        enableInfiniteScroll: false,
                        enlargeCenterPage: true,
                        viewportFraction: 0.5,
                      ),
                      items: snapshot.data!.where((url) => url != null).map((String? url) {
                        if (url!.startsWith("http")) {
                          return Image.network(
                            url,
                            //loadingBuilder: (context, widget, event) => LoadingWidget(),
                          );
                        } else {
                          return Image.file(File(url));
                        }
                      }).toList(),
                    ),
                  ),
                TextButton(
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: <Widget>[
                      Text(
                        NiceLocalizations.of(context).translate("panda.add_picture"),
                      ),
                      const Padding(
                        padding: EdgeInsets.all(5.0),
                        child: Icon(
                          Icons.photo_camera,
                        ),
                      )
                    ],
                  ),
                  onPressed: () async {
                    final image = await PandaPictureUtils.takePicture(context);
                    if (image != null) {
                      _addImage(image);
                    }
                  },
                ),
              ],
            );
          },
        ),
      ],
    );
  }

  void _addImage(File image) async {
    _isLoading.sink.add(true);

    // Add image to the list.
    final fileId = await uploadFile(image);

    if (fileId == null) {
      ScaffoldMessenger.of(context).showSnackBar(SnackBar(
        content: Text(NiceLocalizations.of(context).translate("panda.failed_to_upload_picture")),
      ));
      return;
    }

    LinkedHashMap<String, dynamic>? newNodeValue = node!.value ?? LinkedHashMap.from({"value": null});
    if (newNodeValue["value"] == null) {
      newNodeValue["value"] = [fileId];
    } else {
      (newNodeValue["value"] as List).add(fileId);
    }

    node!.sink.add(newNodeValue);

    final String downloadUrl = await getDownloadUrl(fileId);
    _imageLinks.sink.add([...(_imageLinks.value ?? []), downloadUrl]);

    _isLoading.sink.add(false);
  }

//    void _removeImage(index) async {
//        if (index < 0) {
//            return;
//        }
//
//        _isLoading.sink.add(true);
//
//        // Remove from links.
//        final List<String> imageLinks = _imageLinks.value.toList();
//        if (index < imageLinks.length) {
//            imageLinks.removeAt(index);
//            _imageLinks.sink.add(imageLinks);
//        }
//
//        // Remove from model.
//        LinkedHashMap<String, dynamic> newNodeValue = node.value;
//
//        if (newNodeValue != null &&
//            newNodeValue["value"] != null &&
//            newNodeValue["value"] is List &&
//            index < newNodeValue["value"].length) {
//            newNodeValue["value"].removeAt(index);
//            node.sink.add(newNodeValue);
//        }
//
//        _isLoading.sink.add(false);
//    }
}
